__all__ = [
    "__title__", "__summary__", "__url__", "__version__",
    "__author__", "__email__", "__license__",
]

__title__ = "clikraken"
__summary__ = "Command-line client for the Kraken exchange"
__url__ = "https://github.com/zertrin/clikraken"

__version__ = "0.8.3.2"

__author__ = "Marc Gallet"
__email__ = "zertrin@gmail.com"

__license__ = "Apache License, Version 2.0"
