from clikraken.__about__ import (
    __title__, __summary__, __url__, __version__,
    __author__, __email__, __license__,
)

__all__ = [
    "__title__", "__summary__", "__url__", "__version__",
    "__author__", "__email__", "__license__",
]
